/* 
 * File:   Timer.h
 * Author: Will Matheson
 * Course: PROG 2400 (Data Structures), Section 700
 * Due:    March 25, 2013
 * 
 * A simple timer for POSIX-compatible systems.
 * Uses the monotonic clock that is started when the system boots.
 * This means it's "immune" to subsequent time changes.
 * 
 * Derived from existing work by:
 * CTT and Bobby Powers (stackoverflow.com)
 * alex79roma (cplusplus.com)
 * 
 * http://stackoverflow.com/a/588377
 * http://www.cplusplus.com/forum/beginner/317/
 *
 * Created on February 28, 2013, 5:21 PM
 */

#ifndef TIMER_H
#define	TIMER_H

#include <time.h>

class Timer {
public:
    //big three
    Timer();//constructor (not defined)
    Timer(const Timer& orig);//copy constructor (not defined)
    virtual ~Timer();//virtual destructor (not defined)
    
    //public utilities
    
    //start the timer
    void start();
    
    //find how much time has passed
    unsigned long elapsedTimeSeconds();
    long double elapsedTimeMilliseconds();
    unsigned long elapsedTimeNanoseconds();
    
    //find if we've passed a certain time threshold (useful for games)
    bool isTimeoutSeconds(unsigned long threshold);
    bool isTimeoutMilliseconds(long double threshold);
    bool isTimeoutNanoseconds(unsigned long threshold);
private:
    //when the clock began
    struct timespec _startingTime;
};

#endif	/* TIMER_H */

