/* 
 * File:   LinkedList.cpp
 * Author: Will Matheson
 * Course: PROG 2400 (Data Structures), Section 700
 * Due:    March 25, 2013
 * 
 * Implementation for LinkedList (see header file for description)
 * 
 * Based on earlier LinkedList, which was created January 11, 2013, 3:32 PM
 */

#include "LinkedList.h"

//**BIG THREE**
//default constructor
LinkedList::LinkedList() {
    _head = NULL;//a train without any cars
}

//copy constructor (not defined)
LinkedList::LinkedList(const LinkedList& orig) {
}

//virtual destructor (not defined)
LinkedList::~LinkedList() {
}

//**PUBLIC UTILITIES**
//add a line at the end of the LinkedList
void LinkedList::append(std::string in){
    ListNode* newNode = new ListNode();//create new node
    newNode->setLine(in);//set its data
    if (_head == NULL)//we're just starting up the chain
    {
        _head = newNode;
    }
    else //we're adding onto the chain, so find the end of the chain
    {
        ListNode* runner = _head;
        ListNode* trailer = NULL;
        while(runner != NULL) //while he hasn't fallen off the back of the train
        {
            //move onto the next node
            trailer = runner;
            runner = runner->getNext();
        }
        
        //now, attach the new car to the end of the train
        trailer->setNext(newNode);
    }
}

//delete a line by its index (the LinkedList is 0-indexed)
void LinkedList::remove(int index){
    ListNode* runner = _head;
    ListNode* trailer = NULL;
    if (index < 0)
    {
        std::cerr << "Line must be a non-negative integer." << std::endl;
        return;
    }
    //if index is 0 (start of list)
    else if (index == 0)
        _head = runner->getNext();
    else
    {
        //go to the nth spot (middle or end of list)
        for (int i=0; i<index; i++)
        {
            trailer = runner;
            runner = runner->getNext();
            if (runner == NULL)
            {
                std::cerr << "Line specified for deletion does not exist."
                        << std::endl;
                return;
            }
        }
        
        //"skip" car to be deleted
        trailer->setNext(runner->getNext());
    }    
    
    delete runner;//delete the node from memory
}

//insert a line at an index (the LinkedList is 0-indexed)
void LinkedList::insert(std::string in, int index){
    index ++;//treat the first line as line 1 so we can still insert when empty
    ListNode* newNode = new ListNode();//new node to hold the line
    newNode->setLine(in);//put the line into that new node
    
    ListNode* runner = _head;
    ListNode* trailer = NULL;
    int counter = 0;//for keeping track of where we are
    
    while(runner != NULL)
    {
        //increase counter
        //check if it is location sought
        if (++counter == index)
            break;//stop moving ahead
        trailer = runner;
        runner = runner->getNext();
    }
    
    if(runner != NULL)
    {
        if(trailer == NULL)
        {
            //inserting at beginning
            newNode->setNext(_head);
            _head = newNode;
        }
        else
        {
            //inserting in middle
            //we can't "insert" something to be on the end, use append for that
            newNode->setNext(trailer->getNext());
            trailer->setNext(newNode);                
        }    
    }
    else
    {
        //location doesn't exist
        std::cerr << "Insertion location out of bounds of document." 
                << std::endl;
        return;
    }

}

//count the stored lines
int LinkedList::count(){
    int lines = 0;
    ListNode* node = _head;
    while (node != NULL)//while we haven't fallen off the back of the train
    {
        node = node->getNext();
        lines ++;
    }
    return lines;
}

//displays all of the lines on standard output
//prefixing and indexing are to be specified
void LinkedList::showAllLines(std::string lprefix, int lindex){
    int lnum = lindex;
    ListNode* node = _head;
    while (node != NULL)
    {
        std::cout << lnum << lprefix << node->getLine() << std::endl;//disp line
        node = node->getNext();
        lnum ++;
    }
    return;
}

//displays a range of lines on standard output
//(to display one line, provide the same number for both lbegin and lend)
//as with ShowAllLines, prefixing and indexing are to be specified
void LinkedList::showLines(std::string lprefix, int lindex, int lbegin,
        int lend){
    int lnum = lindex;    
    ListNode* runner = _head;
    
    if (lbegin < 0 || lend < 0)
    {
        std::cerr << "Line numbers must be non-negative integers." << std::endl;
        return;
    }
    else if (lbegin > lend)
    {
        std::cerr << "Finishing index must equal or exceed starting index." 
                << std::endl;
        return;
    }
    else
    {
        //go to the nth spot (middle or end of list)
        for (int i=0; i<lbegin; i++)
        {
            runner = runner->getNext();
            lnum++;
            if (runner == NULL)
            {
                std::cerr << "Line specified for listing does not exist."
                        << std::endl;
                return;
            }
        }
        
        //display the first line
        std::cout << lnum << lprefix << runner->getLine() << std::endl;
        
        //go to the nth+nth spot and display the lines along the way
        for (int i=0; i<(lend-lbegin); i++)
        {
            runner = runner->getNext();
            lnum++;
            if (runner == NULL)
            {
                std::cerr << "Line specified for listing does not exist." 
                        << std::endl;
                return;
            }
            else
            {
                //display the nth line
                std::cout << lnum << lprefix << runner->getLine() << std::endl;
            }                
        }
    }    
    

}

//stream entire contents
std::ostream& operator<<(std::ostream& output, LinkedList& list){
    ListNode* node = list._head;
    while (node != NULL)
    {
        output << node->getLine();
        node = node->getNext();
        if (node != NULL)//so we're not adding a new line on the bottom
           output << std::endl;
    }
    return output;
}
